using System;
using System.Data;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;

using CodePlex.XmlDataMapper.ConsoleClient.Dto;
using CodePlex.XmlDataMapper.ConsoleClient.Bll;

namespace CodePlex.XmlDataMapper.ConsoleClient
{
   [TestFixture(Description = "Testing convertions")]
   public static class OrderRun
   {
      [Test(Description = "Testing DTOConverter<T>.Convert(DataTable) sucess scenario")]
      public static void TestDataTableToDTOConversion()
      {
         DataTable dataTable = OrderRandomizer.GetOrdersDataTable(20); //Get Random data in DataTable
         
         IList<Order> list = DTOConverter<Order>.Convert(dataTable); //This is `it`
         
         Console.WriteLine(" ---------TestDataTableToDTOConversion ------------");
         Array.ForEach(((List<Order>)list).ToArray(), WriteHost);

         //NUnit Unit Testing code
         Assert.IsTrue(dataTable.Rows.Count == list.Count, "Not all rows were converted");
         Assert.IsTrue(dataTable.Rows[0]["Name"].ToString() == list[0].Name, "First Column Value Mismatch");
      }

      [Test(Description = "Testing DTOConverter<T>.Convert(IDataReader) sucess scenario")]
      public static void TestDataReaderToDTOConversion()
      {
         IDataReader reader = OrderRandomizer.GetOrdersDataReader(20); //Get Random data in data reader

         IList<Order> list = DTOConverter<Order>.Convert(reader); //This is `it` 

         Console.WriteLine(" ---------TestDataReaderToDTOConversion ------------");
         Array.ForEach(((List<Order>)list).ToArray(), WriteHost);

         //NUnit Unit Testing code
         Assert.IsTrue(list != null && list.Count > 0, "Nothing was converted, empty IList returned");
      }

      private static void WriteHost(Order order)
      {
         Console.WriteLine("Id:{0} \t Name:{1} \t Amount:{2}", order.Id, order.Name, order.Amount);
      }

      //Your actual unit testing code may look like this
      //Will not work unless you have a db instance created with structure, SPs and data : Coming soon
      [Ignore("To test OrderAction.GetAll() sucess scenario, but database example not fully ready yet so ignoring")]
      public static void TestDataTableToDTOConversionActual()
      {
         IList<Order> list = OrderAction.GetAll(); //Actually fetches Order records from the Orders table in the DataBase
         Console.WriteLine(" ---------TestDataTableToDTOConversionActual------------");
         Array.ForEach(((List<Order>)list).ToArray(), WriteHost);
         Assert.IsTrue(list != null && list.Count > 0, "Nothing was retrieved from Db or conversion failed, empty IList returned");
      }
   }
}
//Copyright Binoj V Antony  2010 under GNU Library General Public License (LGPL) at http://XmlDataMapper.CodePlex.com/
//Please preserve the authors names and copyright statement in the code even when the work is changed or added to.