using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Data;
using System.Data.Common;
using System.Diagnostics;
using System.Globalization;
using System.Text;

using Microsoft.Practices.EnterpriseLibrary.Data;

namespace CodePlex.XmlDataMapper.Helpers
{
   public class BaseDataHelper : IDisposable
   {
      #region Constructor and Fields

      public BaseDataHelper() {/*no-op*/}

      public BaseDataHelper(string databaseConnectionFromConfiguration)
      {
         this.db = DatabaseFactory.CreateDatabase(databaseConnectionFromConfiguration);
      }

      public BaseDataHelper(bool enlistInTransaction)
      {
         this.enlistInTransaction = enlistInTransaction;
      }

      private Database db = null;
      private DbConnection dbConnection = null;
      private DbCommand dbCommand = null;
      private DbTransaction dbTransaction = null;
      private bool enlistInTransaction;
      private bool disposed;

      #endregion

      #region Instance Methods

      public void Commit()
      {
         this.dbTransaction.Commit();
      }

      public void Rollback(bool supressError)
      {
         try
         {
            this.dbTransaction.Rollback();
         }
         catch (Exception ex)
         {
            if (supressError == false) throw;
         }
      }

      public int ExecuteNonQueryInstance(string storedProcedureName, List<GenericDbParameter> paramList, bool supressError)
      {
         try
         {
            if (this.db == null) this.db = DatabaseFactory.CreateDatabase();
            if (this.dbConnection == null) this.dbConnection = this.db.CreateConnection();
            if (this.dbConnection.State == ConnectionState.Closed) this.dbConnection.Open();
            if (this.dbCommand == null) this.dbCommand = this.dbConnection.CreateCommand();

            if (this.enlistInTransaction)
            {
               if (this.dbTransaction == null) this.dbTransaction = this.dbConnection.BeginTransaction(IsolationLevel.ReadUncommitted);
               if (this.dbCommand.Transaction == null) this.dbCommand.Transaction = this.dbTransaction;
            }

            this.dbCommand.CommandText = storedProcedureName;
            this.dbCommand.CommandType = CommandType.StoredProcedure;
            this.dbCommand.Parameters.Clear();
            AddParameters(this.dbCommand, paramList);

            int recordsAffected = 0;
            if (this.enlistInTransaction)
            {
               recordsAffected = this.db.ExecuteNonQuery(this.dbCommand, this.dbTransaction);
            }
            else
            {
               recordsAffected = this.db.ExecuteNonQuery(this.dbCommand);
            }
            FillOutParameter(paramList);
            return recordsAffected;
         }
         catch (Exception ex)
         {
            if (this.enlistInTransaction) this.dbTransaction.Rollback();
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return -1;
         }
      }

      public object ExecuteScalarInstance(string storedProcedureName, List<GenericDbParameter> paramList, bool supressError)
      {
         try
         {
            if (this.db == null) this.db = DatabaseFactory.CreateDatabase();
            if (this.dbConnection == null) this.dbConnection = this.db.CreateConnection();
            if (this.dbConnection.State == ConnectionState.Closed) this.dbConnection.Open();
            if (this.dbCommand == null) this.dbCommand = this.dbConnection.CreateCommand();
            if (this.enlistInTransaction)
            {
               if (this.dbTransaction == null) this.dbTransaction = this.dbConnection.BeginTransaction(IsolationLevel.ReadUncommitted);
               //To change/make configurable later
               if (this.dbCommand.Transaction == null) this.dbCommand.Transaction = this.dbTransaction;
            }

            this.dbCommand.CommandText = storedProcedureName;
            this.dbCommand.CommandType = CommandType.StoredProcedure;
            this.dbCommand.Parameters.Clear();
            AddParameters(this.dbCommand, paramList);
            object returnValue = null;
            if (this.enlistInTransaction)
            {
               returnValue = this.db.ExecuteScalar(this.dbCommand, this.dbTransaction);
            }
            else
            {
               returnValue = this.db.ExecuteScalar(this.dbCommand);
               //returnValue = Me.dbCommand.ExecuteScalar()
            }
            FillOutParameter(paramList);
            return returnValue;
         }
         catch (Exception ex)
         {
            if (this.enlistInTransaction) this.dbTransaction.Rollback();
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return null;
         }
      }

      public IDataReader ExecuteReaderInstance(string storedProcedureName, List<GenericDbParameter> paramList, bool supressError)
      {
         try
         {
            if (this.db == null) this.db = DatabaseFactory.CreateDatabase();
            if (this.dbConnection == null) this.dbConnection = this.db.CreateConnection();
            if (this.dbConnection.State == ConnectionState.Closed) this.dbConnection.Open();
            if (this.dbCommand == null) this.dbCommand = this.dbConnection.CreateCommand();
            if (this.enlistInTransaction)
            {
               if (this.dbTransaction == null) this.dbTransaction = this.dbConnection.BeginTransaction(IsolationLevel.ReadUncommitted);
               if (this.dbCommand.Transaction == null) this.dbCommand.Transaction = this.dbTransaction;
            }

            this.dbCommand.CommandText = storedProcedureName;
            this.dbCommand.CommandType = CommandType.StoredProcedure;
            this.dbCommand.Parameters.Clear();
            AddParameters(this.dbCommand, paramList);
            IDataReader dataReader = null;
            if (this.enlistInTransaction)
            {
               dataReader = this.db.ExecuteReader(this.dbCommand, this.dbTransaction);
            }
            else
            {
               dataReader = this.db.ExecuteReader(this.dbCommand);
               //dataReader = Me.dbCommand.ExecuteReader()
            }
            FillOutParameter(paramList);
            return dataReader;
         }
         catch (Exception ex)
         {
            if (this.enlistInTransaction) this.dbTransaction.Rollback();
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return null;
         }
      }

      public int UpdateDataSetInstance(string storedProcedureName, List<GenericDbParameter> paramList, DataSet dataSet, bool supressError)
      {
         try
         {
            if (this.db == null) this.db = DatabaseFactory.CreateDatabase();
            if (this.dbConnection == null) this.dbConnection = this.db.CreateConnection();
            if (this.dbConnection.State == ConnectionState.Closed) this.dbConnection.Open();
            if (this.dbCommand == null) this.dbCommand = this.dbConnection.CreateCommand();

            this.dbCommand.CommandText = storedProcedureName;
            this.dbCommand.CommandType = CommandType.StoredProcedure;
            this.dbCommand.Parameters.Clear();

            if (this.enlistInTransaction)
            {
               if (this.dbTransaction == null) this.dbTransaction = this.dbConnection.BeginTransaction(IsolationLevel.ReadUncommitted);
               if (this.dbCommand.Transaction == null) this.dbCommand.Transaction = this.dbTransaction;
            }

            AddParametersForDataSetUpdate(this.dbCommand, paramList);
            if (this.enlistInTransaction)
            {
               return this.db.UpdateDataSet(dataSet, dataSet.Tables[0].TableName, this.dbCommand, null, null, this.dbTransaction);
            }
            else
            {
               return this.db.UpdateDataSet(dataSet, dataSet.Tables[0].TableName, this.dbCommand, null, null, UpdateBehavior.Standard);
            }
         }
         catch (Exception ex)
         {
            if (this.enlistInTransaction) this.dbTransaction.Rollback();
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return -1;
         }
      }

      public void FillOutParameter(List<GenericDbParameter> paramList)
      {
         FillOutParameter(this.dbCommand, paramList);
      }

      #endregion

      #region Loading or Executing DataSet

      public static void LoadDataSet(string storedProcedureName, string tableName, DataSet dataSet, List<GenericDbParameter> paramList, bool supressError)
      {
         LoadDataSet(storedProcedureName, new string[] { tableName }, dataSet, paramList, supressError);
      }

      public static void LoadDataSet(string storedProcedureName, string[] tableNames, DataSet dataSet, List<GenericDbParameter> paramList, bool supressError)
      {
         Database db = null;
         DbCommand dbCommand = null;
         try
         {
            db = DatabaseFactory.CreateDatabase();
            dbCommand = db.GetStoredProcCommand(storedProcedureName);
            AddParameters(dbCommand, paramList);
            db.LoadDataSet(dbCommand, dataSet, tableNames);
         }
         catch (Exception ex)
         {
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
         }
         finally
         {
            if (dbCommand != null) dbCommand.Dispose();
            dbCommand = null;
            db = null;
         }
      }
      public static DataSet ExecuteDataSet(string storedProcedureName, string tableName, List<GenericDbParameter> paramList, bool supressError)
      {
         return ExecuteDataSet(storedProcedureName, new string[] { tableName }, paramList, supressError);
      }

      public static DataSet ExecuteDataSet(string storedProcedureName, string[] tableNames, List<GenericDbParameter> paramList, bool supressError)
      {
         Database db = null;
         DbCommand dbCommand = null;
         DataSet dataSet = new DataSet();
         dataSet.Locale = CultureInfo.InvariantCulture;
         try
         {
            db = DatabaseFactory.CreateDatabase();
            dbCommand = db.GetStoredProcCommand(storedProcedureName);
            AddParameters(dbCommand, paramList);
            db.LoadDataSet(dbCommand, dataSet, tableNames);
         }
         catch (Exception ex)
         {
            TraceError(ex, storedProcedureName);
            dataSet.Dispose();
            dataSet = null;
            if (supressError == false) throw;
         }
         finally
         {
            if (dbCommand != null) dbCommand.Dispose();
            dbCommand = null;
            db = null;
         }
         return dataSet;
      }

      #endregion

      #region DataReader

      public static IDataReader ExecuteReader(string storedProcedureName, List<GenericDbParameter> paramList, bool supressError)
      {
         Database db = null;
         DbCommand dbCommand = null;
         try
         {
            db = DatabaseFactory.CreateDatabase();
            dbCommand = db.GetStoredProcCommand(storedProcedureName);
            AddParameters(dbCommand, paramList);
            IDataReader dataReader = db.ExecuteReader(dbCommand);
            FillOutParameter(dbCommand, paramList);
            return dataReader;
         }
         catch (Exception ex)
         {
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return null;
         }
         finally
         {
            if (dbCommand != null) dbCommand.Dispose();
            dbCommand = null;
            db = null;
         }
      }

      #endregion

      #region FieldMapped Query Execution

      // may be redundant
      public static int ExecuteNonQuery(string storedProcedureName, FieldMappers fieldMappers, DataRow dataRow, bool supressError)
      {
         List<GenericDbParameter> paramList = new List<GenericDbParameter>();
         LoadParams(dataRow, fieldMappers, paramList);
         return ExecuteNonQuery(storedProcedureName, paramList, supressError);
      }

      public static int ExecuteNonQuery(string storedProcedureName, FieldMappers fieldMappers, DataRow dataRow, SqlOperation sqlOperation, bool supressError)
      {
         List<GenericDbParameter> paramList = new List<GenericDbParameter>();
         if (sqlOperation == SqlOperation.Insert)
         {
            LoadParamsForInsert(dataRow, fieldMappers, paramList);
         }
         else if (sqlOperation == SqlOperation.Update)
         {
            LoadParamsForUpdate(dataRow, fieldMappers, paramList);
         }
         return ExecuteNonQuery(storedProcedureName, paramList, supressError);
      }

      public static int UpdateDataSet(string storedProcedureName, FieldMappers fieldMappers, DataSet dataSet, SqlOperation sqlOperation, bool supressError)
      {
         Database db = null;
         DbCommand dbCommand = null;
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         int recordsAffected = 0;
         try
         {
            db = DatabaseFactory.CreateDatabase();
            dbCommand = db.GetStoredProcCommand(storedProcedureName);
            if (sqlOperation == SqlOperation.Insert)
            {
               for (int i = 0; i <= columnCount; i++)
               {
                  if (fieldMappers.FieldMapper[i].Insert == true && fieldMappers.FieldMapper[i].Optional == false)
                  {
                     db.AddInParameter(dbCommand, fieldMappers.FieldMapper[i].Property, fieldMappers.FieldMapper[i].DbType, fieldMappers.FieldMapper[i].ColumnName, DataRowVersion.Default);
                  }
               }
            }
            else
            {
               for (int i = 0; i <= columnCount; i++)
               {
                  if (fieldMappers.FieldMapper[i].Update == true && fieldMappers.FieldMapper[i].Optional == false)
                  {
                     db.AddInParameter(dbCommand, fieldMappers.FieldMapper[i].Property, fieldMappers.FieldMapper[i].DbType, fieldMappers.FieldMapper[i].ColumnName, DataRowVersion.Default);
                  }
               }
            }
            recordsAffected = db.UpdateDataSet(dataSet, dataSet.Tables[0].TableName, dbCommand, null, null, UpdateBehavior.Standard);
         }
         catch (Exception ex)
         {
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return -1;
         }
         finally
         {
            if (dbCommand != null) dbCommand.Dispose();
            dbCommand = null;
            db = null;
         }
         return recordsAffected;
      }

      public static int ExecuteMasterDetailInsert(string[] storedProcedureNames, List<GenericDbParameter> paramList, FieldMappers fieldMappers, string ParentId, DataSet dataSet, bool supressError)
      {
         Database db = null;
         DbCommand dbCommand = null;
         DbCommand dbCommand2 = null;
         object returnValue = null;
         DbConnection dbConnnection = null;
         DbTransaction dbTransaction = null;
         int recordsAffected = 0;
         try
         {
            db = DatabaseFactory.CreateDatabase();
            dbConnnection = db.CreateConnection();
            dbConnnection.Open();
            dbTransaction = dbConnnection.BeginTransaction(IsolationLevel.ReadUncommitted);
            dbCommand = db.GetStoredProcCommand(storedProcedureNames[0]);
            dbCommand.Transaction = dbTransaction;
            AddParameters(dbCommand, paramList);
            returnValue = db.ExecuteScalar(dbCommand, dbTransaction);

            int rowCount = dataSet.Tables[0].Rows.Count - 1;
            for (int i = 0; i <= rowCount; i++)
            {
               dataSet.Tables[0].Rows[i][ParentId] = returnValue;
            }

            dbCommand2 = db.GetStoredProcCommand(storedProcedureNames[1]);

            int columnCount = fieldMappers.FieldMapper.Length - 1;
            for (int i = 0; i <= columnCount; i++)
            {
               if (fieldMappers.FieldMapper[i].Insert == true && fieldMappers.FieldMapper[i].Optional == false)
               {
                  db.AddInParameter(dbCommand2, fieldMappers.FieldMapper[i].Property, fieldMappers.FieldMapper[i].DbType, fieldMappers.FieldMapper[i].ColumnName, DataRowVersion.Default);
               }
            }

            recordsAffected = db.UpdateDataSet(dataSet, dataSet.Tables[0].TableName, dbCommand2, null, null, dbTransaction);
            dbTransaction.Commit();
         }
         catch (Exception ex)
         {
            dbTransaction.Rollback();
            TraceError(ex, storedProcedureNames[0] + " & " + storedProcedureNames[1]);
            if (supressError == false) throw;
            return -1;
         }
         finally
         {
            if (dbCommand != null) dbCommand.Dispose();
            dbCommand = null;
            if (dbCommand2 != null) dbCommand2.Dispose();
            dbCommand2 = null;
            if (dbTransaction != null) dbTransaction.Dispose();
            dbTransaction = null;
            if (dbConnnection != null) dbConnnection.Dispose();
            dbConnnection = null;
         }
         return recordsAffected;
      }

      #endregion

      #region Non Query and Scalar

      public static int ExecuteNonQuery(string storedProcedureName, List<GenericDbParameter> paramList, bool supressError)
      {
         Database db = null;
         DbCommand dbCommand = null;
         try
         {
            db = DatabaseFactory.CreateDatabase();
            dbCommand = db.GetStoredProcCommand(storedProcedureName);
            AddParameters(dbCommand, paramList);
            int recordsAffected = db.ExecuteNonQuery(dbCommand);
            FillOutParameter(dbCommand, paramList);
            return recordsAffected;
         }
         catch (Exception ex)
         {
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return -1;
         }
         finally
         {
            if (dbCommand != null) dbCommand.Dispose();
            dbCommand = null;
            db = null;
         }
      }

      public static int ExecuteNonQuery(string storedProcedureName, List<GenericDbParameter> paramList, DbConnection dbConnnection, DbTransaction dbTransaction, bool supressError)
      {
         DbCommand dbCommand = null;
         try
         {
            dbCommand = dbConnnection.CreateCommand();
            dbCommand.CommandType = CommandType.StoredProcedure;
            dbCommand.CommandText = storedProcedureName;
            dbCommand.Transaction = dbTransaction;
            AddParameters(dbCommand, paramList);
            int recordsAffected = dbCommand.ExecuteNonQuery();
            FillOutParameter(dbCommand, paramList);
            dbTransaction.Commit();
            return recordsAffected;
         }
         catch (Exception ex)
         {
            dbTransaction.Rollback();
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return -1;
         }
         finally
         {
            if (dbCommand != null) dbCommand.Dispose();
            dbCommand = null;
         }
      }

      public static object ExecuteScalar(string storedProcedureName, List<GenericDbParameter> paramList, bool supressError)
      {
         Database db = null;
         DbCommand dbCommand = null;
         try
         {
            db = DatabaseFactory.CreateDatabase();
            dbCommand = db.GetStoredProcCommand(storedProcedureName);
            AddParameters(dbCommand, paramList);
            object returnValue = db.ExecuteScalar(dbCommand);
            FillOutParameter(dbCommand, paramList);
            return returnValue;
         }
         catch (Exception ex)
         {
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return null;
         }
         finally
         {
            if (dbCommand != null) dbCommand.Dispose();
            dbCommand = null;
            db = null;
         }
      }

      #endregion

      #region Update DataSet

      public static int UpdateDataSet(string storedProcedureName, List<GenericDbParameter> paramList, DataSet dataSet, bool supressError)
      {
         Database db = null;
         DbCommand dbCommand = null;
         int recordsAffected = 0;
         try
         {
            db = DatabaseFactory.CreateDatabase();
            dbCommand = db.GetStoredProcCommand(storedProcedureName);
            AddParametersForDataSetUpdate(dbCommand, paramList);
            recordsAffected = db.UpdateDataSet(dataSet, dataSet.Tables[0].TableName, dbCommand, null, null, UpdateBehavior.Standard);
            FillOutParameter(dbCommand, paramList);
         }
         catch (Exception ex)
         {
            TraceError(ex, storedProcedureName);
            if (supressError == false) throw;
            return -1;
         }
         finally
         {
            if (dbCommand != null) dbCommand.Dispose();
            dbCommand = null;
            db = null;
         }
         return recordsAffected;
      }
      #endregion

      #region FieldMappers - Parameter addition helpers

      public static void LoadParams(FieldMappers fieldMappers, List<GenericDbParameter> paramList)
      {
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         for (int i = 0; i <= columnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            if (fieldMapper.Optional == false)
            {
               paramList.Add(new GenericDbParameter(fieldMapper.ColumnName, fieldMapper.ColumnName, fieldMapper.DbType, fieldMapper.Bytes));
            }
         }
      }

      public static void LoadParams(DataRow dataRow, FieldMappers fieldMappers, List<GenericDbParameter> paramList)
      {
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         for (int i = 0; i <= columnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            if (fieldMapper.Optional == false)
            {
               paramList.Add(new GenericDbParameter(fieldMapper.ColumnName, fieldMapper.DbType, fieldMapper.Bytes, dataRow[fieldMapper.ColumnName]));
            }
         }
      }

      public static void LoadParamsForInsert(FieldMappers fieldMappers, List<GenericDbParameter> paramList)
      {
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         for (int i = 0; i <= columnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            if (fieldMapper.Insert == true && fieldMapper.Optional == false)
            {
               paramList.Add(new GenericDbParameter(fieldMapper.ColumnName, fieldMapper.ColumnName, fieldMapper.DbType, fieldMapper.Bytes));
            }
         }
      }

      public static void LoadParamsForInsert(DataRow dataRow, FieldMappers fieldMappers, List<GenericDbParameter> paramList)
      {
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         for (int i = 0; i <= columnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            if (fieldMapper.Insert == true && fieldMapper.Optional == false)
            {
               paramList.Add(new GenericDbParameter(fieldMapper.ColumnName, fieldMapper.DbType, fieldMapper.Bytes, dataRow[fieldMapper.ColumnName]));
            }
         }
      }

      public static void LoadParamsForUpdate(FieldMappers fieldMappers, List<GenericDbParameter> paramList)
      {
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         for (int i = 0; i <= columnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            if (fieldMapper.Update == true && fieldMapper.Optional == false)
            {
               paramList.Add(new GenericDbParameter(fieldMapper.ColumnName, fieldMapper.ColumnName, fieldMapper.DbType, fieldMapper.Bytes));
            }
         }
      }

      public static void LoadParamsForUpdate(Type type, List<GenericDbParameter> paramList)
      {
         FieldMappers fieldMappers = CacheHelper.GetFromCache(type);
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         for (int i = 0; i <= columnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            if (fieldMapper.Update == true && fieldMapper.Optional == false)
            {
               paramList.Add(new GenericDbParameter(fieldMapper.ColumnName, fieldMapper.ColumnName, fieldMapper.DbType, fieldMapper.Bytes));
            }
         }
      }

      public static void LoadParamsForUpdate(DataRow dataRow, FieldMappers fieldMappers, List<GenericDbParameter> paramList)
      {
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         for (int i = 0; i <= columnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            if (fieldMapper.Update == true && fieldMapper.Optional == false)
            {
               paramList.Add(new GenericDbParameter(fieldMapper.ColumnName, fieldMapper.DbType, fieldMapper.Bytes, dataRow[fieldMapper.ColumnName]));
            }
         }
      }

      public static void LoadParamsForDTOInsert(object dto, List<GenericDbParameter> paramList)
      {
         Type type = dto.GetType();
         FieldMappers fieldMappers = CacheHelper.GetFromCache(type);
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         for (int i = 0; i <= columnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            if (fieldMapper.Insert == true && fieldMapper.Optional == false)
            {
               object value = type.GetProperty(fieldMapper.Property).GetValue(dto, null);
               if (value == null)
               {
                  paramList.Add(new GenericDbParameter(fieldMapper.ColumnName, fieldMapper.DbType, fieldMapper.Bytes, DBNull.Value));
               }
               else
               {
                  paramList.Add(new GenericDbParameter(fieldMapper.ColumnName, fieldMapper.DbType, fieldMapper.Bytes, type.GetProperty(fieldMapper.Property).GetValue(dto, null)));
               }
            }
         }
      }

      //To be tested
      public static void LoadParamsForDTOUpdate(object dto, List<GenericDbParameter> paramList)
      {
         Type type = dto.GetType();
         FieldMappers fieldMappers = CacheHelper.GetFromCache(type);
         int columnCount = fieldMappers.FieldMapper.Length - 1;
         for (int i = 0; i <= columnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            if (fieldMapper.Update == true && fieldMapper.Optional == false)
            {
               paramList.Add(new GenericDbParameter(fieldMapper.Property, fieldMapper.DbType, fieldMapper.Bytes, type.GetProperty(fieldMapper.Property).GetValue(dto, null)));
            }
         }
      }

      #endregion

      #region Parameter addition helpers

      public static void AddParameters(DbCommand dbCommand, List<GenericDbParameter> paramList)
      {
         if (paramList == null) return;
         int count = paramList.Count;
         List<DbParameter> dbParamList = new List<DbParameter>(count);
         for (int i = 0; i <= count - 1; i++)
         {
            GenericDbParameter myDbParameter = paramList[i];
            DbParameter dbParameter = dbCommand.CreateParameter();
            dbParameter.Direction = myDbParameter.Direction;
            dbParameter.ParameterName = myDbParameter.Name;
            dbParameter.DbType = myDbParameter.DbType;
            if (dbParameter.Size > 0) dbParameter.Size = myDbParameter.Size;
            dbParameter.Value = myDbParameter.Value;
            dbParamList.Add(dbParameter);
         }
         dbCommand.Parameters.AddRange(dbParamList.ToArray());
      }

      public static void FillOutParameter(DbCommand dbCommand, List<GenericDbParameter> paramList)
      {
         if (dbCommand == null || paramList == null) return;
         int count = paramList.Count - 1;
         for (int i = 0; i <= count; i++)
            if (paramList[i].Direction == ParameterDirection.Output)
               paramList[i].Value = dbCommand.Parameters[i].Value;
      }

      public static void AddParametersForDataSetUpdate(DbCommand dbCommand, List<GenericDbParameter> paramList)
      {
         if (paramList == null) return;

         int count = paramList.Count;
         List<DbParameter> dbParamList = new List<DbParameter>(count);
         for (int i = 0; i < count; i++)
         {
            GenericDbParameter myDbParameter = paramList[i];
            DbParameter dbParameter = dbCommand.CreateParameter();
            dbParameter.Direction = myDbParameter.Direction;
            dbParameter.ParameterName = myDbParameter.Name;
            dbParameter.DbType = myDbParameter.DbType;
            dbParameter.SourceColumn = myDbParameter.ColumnName;
            if (dbParameter.Size > 0) dbParameter.Size = myDbParameter.Size;
            dbParameter.SourceVersion = DataRowVersion.Default;
            dbParamList.Add(dbParameter);
         }
         dbCommand.Parameters.AddRange(dbParamList.ToArray());
      }

      #endregion

      #region Trace Error Method

      public static void TraceError(Exception e, string statementName)
      {
         if (e == null)
         {
            return;
         }
         if (!string.IsNullOrEmpty(statementName))
         {
            Trace.WriteLine("CodePlex.XmlDataMapper:StatemementName:" + statementName);
         }
         Trace.WriteLine("CodePlex.XmlDataMapper:Error:" + e.ToString());
      }

      #endregion

      #region IDisposable Implementation

      public void Dispose()
      {
         this.Dispose(true);
         GC.SuppressFinalize(this);
      }

      protected virtual void Dispose(bool disposing)
      {
         if (this.disposed) return;
         if (this.dbConnection != null) this.dbConnection.Dispose();
         this.dbConnection = null;
         if (disposing)
         {
            if (this.dbCommand != null) this.dbCommand.Dispose();
            if (this.dbTransaction != null) this.dbTransaction.Dispose();
            this.dbCommand = null;
            this.dbTransaction = null;
            this.db = null;
         }
         this.disposed = true;
      }

      ~BaseDataHelper()
      {
         this.Dispose(false);
      }

      #endregion
   }
}
//Copyright Binoj V Antony  2010 under GNU Library General Public License (LGPL) at http://XmlDataMapper.CodePlex.com/
//Please preserve the authors names and copyright statement in the code even when the work is changed or added to.