using System;
using System.Collections.Generic;
using System.Collections;
using System.Data;
using System.Data.Common;
using System.Text;
using System.Xml.Serialization;

namespace CodePlex.XmlDataMapper.Helpers
{
   [Serializable]
   public class FieldMapper
   {
      #region constructors

      public FieldMapper() {/*no-op*/}

      public FieldMapper(FieldMapper sourceToCopy)
      {
         this.property = sourceToCopy.property;
         this.columnName = sourceToCopy.columnName;
         this.defaultValue = sourceToCopy.defaultValue;
         this.customData = sourceToCopy.customData;
         this.customDataXmlElement = sourceToCopy.customDataXmlElement;
         this.maxLength = sourceToCopy.maxLength;
         this.bytes = sourceToCopy.bytes;
         this.optional = sourceToCopy.optional;
         this.isPrimaryKey = sourceToCopy.isPrimaryKey;
         this.isIdentity = sourceToCopy.isIdentity;
         this.insert = sourceToCopy.insert;
         this.update = sourceToCopy.update;
         this.typeName = sourceToCopy.typeName;
         this.dbType = sourceToCopy.dbType;
         this.dbColumnType = sourceToCopy.dbColumnType;
         this.dbTypeName = sourceToCopy.dbTypeName;
      }

      #endregion

      #region private fields

      private string property;
      private string columnName;
      private string defaultValue;
      private string customData;
      private string customDataXmlElement;
      private int maxLength = 1;
      private int bytes = 1;
      //private byte ordinal = 255;
      private bool optional = false;
      private bool isPrimaryKey = false;
      private bool isIdentity = false;
      private bool insert = true;
      private bool update = true;

      private string typeName = "System.String";
      private DbType dbType = DbType.String;
      private string dbColumnType;
      private string dbTypeName;

      #endregion

      #region public properties

      [XmlAttribute]
      public string Property
      {
         get { return this.property; }
         set { this.property = value; }
      }

      [XmlAttribute]
      public string ColumnName
      {
         get { return this.columnName; }
         set { this.columnName = value; }
      }

      [XmlAttribute("TypeName")]
      public string TypeName
      {
         get { return this.typeName; }
         set { this.typeName = value; }
      }

      [XmlIgnore]
      public Type Type
      {
         get { return Type.GetType(this.typeName, false, true); }
      }

      [XmlAttribute]
      public string DbColumnType
      {
         get { return this.dbColumnType; }
         set { this.dbColumnType = value; }
      }

      [XmlAttribute("DbType")]
      public string DbTypeName
      {
         get 
         { 
            return this.dbTypeName;
         }

         set
         {
            this.dbTypeName = value;
            if (Enum.IsDefined(typeof(DbType), value))
               this.dbType = (DbType)Enum.Parse(typeof(DbType), value, true);
         }
      }

      [XmlIgnore]
      public DbType DbType
      {
         get { return this.dbType; }
         set { this.dbType = value; }
      }

      [XmlAttribute]
      public string DefaultValue
      {
         get { return this.defaultValue; }
         set { this.defaultValue = value; }
      }

      [XmlAttribute]
      public string CustomData
      {
         get { return this.customData; }
         set { this.customData = value; }
      }

      [XmlElement]
      public string CustomDataXmlElement
      {
         get { return this.customDataXmlElement; }
         set { this.customDataXmlElement = value; }
      }

      [XmlAttribute]
      public int MaxLength
      {
         get { return this.maxLength; }
         set { this.maxLength = value; }
      }

      [XmlAttribute]
      public int Bytes
      {
         get { return this.bytes; }
         set { this.bytes = value; }
      }

      [XmlAttribute]
      public bool Optional
      {
         get { return this.optional; }
         set { this.optional = value; }
      }

      [XmlAttribute]
      public bool Insert
      {
         get { return this.insert; }
         set { this.insert = value; }
      }

      [XmlAttribute]
      public bool Update
      {
         get { return this.update; }
         set { this.update = value; }
      }

      [XmlAttribute]
      public bool IsPrimaryKey
      {
         get { return this.isPrimaryKey; }
         set { this.isPrimaryKey = value; }
      }

      [XmlAttribute]
      public bool IsIdentity
      {
         get { return this.isIdentity; }
         set { this.isIdentity = value; }
      }

      #endregion
   }
}
//Copyright Binoj V Antony  2010 under GNU Library General Public License (LGPL) at http://XmlDataMapper.CodePlex.com/
//Please preserve the authors names and copyright statement in the code even when the work is changed or added to.