using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;

namespace CodePlex.XmlDataMapper.Helpers
{
   public class GenericDbParameter
   {
      private string name;
      private string columnName;
      private object value;
      private DbType dbType;
      private ParameterDirection direction = ParameterDirection.Input;
      private Type type;
      private int size = 0;

      public GenericDbParameter(string name, DbType dbType, object value)
      {
         this.Name = name;
         this.dbType = dbType;
         this.Value = value;
      }

      public GenericDbParameter(string name, DbType dbType, int size, object value)
      {
         this.Name = name;
         this.dbType = dbType;
         this.size = size;
         this.Value = value;
      }

      public GenericDbParameter(string name, DbType dbType, int size, object value, ParameterDirection direction)
      {
         this.Name = name;
         this.dbType = dbType;
         this.size = size;
         this.Value = value;
         this.Direction = direction;
      }

      public GenericDbParameter(string name, Type type, object value)
      {
         this.Name = name;
         this.type = type;
         this.Value = value;
      }

      public GenericDbParameter(string name, string columnName, Type type)
      {
         this.Name = name;
         this.ColumnName = columnName;
         this.type = type;
      }

      public GenericDbParameter(string name, string columnName, DbType dbType, int size)
      {
         this.Name = name;
         this.ColumnName = columnName;
         this.dbType = dbType;
      }

      public GenericDbParameter(string name, string columnName, DbType dbType, int size, ParameterDirection direction)
      {
         this.Name = name;
         this.ColumnName = columnName;
         this.dbType = dbType;
         this.size = size;
         this.Direction = direction;
      }

      public GenericDbParameter(string name, object value)
      {
         this.Name = name;
         this.Value = value;
         this.type = typeof(string);
      }

      public string Name 
      {
         get { return this.name; }
         set { this.name = value; }
      }

      public string ColumnName
      {
         get { return this.columnName; }
         set { this.columnName = value; }
      }

      public ParameterDirection Direction
      {
         get { return this.direction; }
         set { this.direction = value; }
      }

      public DbType DbType
      {
         get { return this.dbType; }
         set { this.dbType = value; }
      }

      public Type Type
      {
         get { return this.type; }
         set
         {
            this.dbType = Convert(value);
            this.type = value;
         }
      }

      public object Value
      {
         get { return this.value; }
         set { this.value = value; }
      }

      public int Size
      {
         get { return this.size; }
         set { this.size = value; }
      }

      public static DbType Convert(Type dataType)
      {
         if (dataType == typeof(System.String)) return DbType.String;
         if (dataType == typeof(System.Int32)) return DbType.Int32;
         if (dataType == typeof(System.Int16)) return DbType.Int16;
         if (dataType == typeof(System.Boolean)) return DbType.Boolean;
         if (dataType == typeof(System.Byte)) return DbType.Byte;
         if (dataType == typeof(System.DateTime)) return DbType.DateTime;
         if (dataType == typeof(System.Decimal)) return DbType.Decimal;
         if (dataType == typeof(System.Int64)) return DbType.Int64;
         if (dataType == typeof(System.Object)) return DbType.Object;

         return DbType.String;
      }
   }
}
//Copyright Binoj V Antony  2010 under GNU Library General Public License (LGPL) at http://XmlDataMapper.CodePlex.com/
//Please preserve the authors names and copyright statement in the code even when the work is changed or added to.